from typing import List, Any, Optional
import pprint
import logging
from langchain_core.runnables import Runnable
from langchain_core.messages import ToolMessage
from langgraph.prebuilt.tool_executor import ToolExecutor, ToolInvocation
from graphs.codegen.state import AgentState
from ..utls import log


def create_agent_executor(chain: Runnable, messages_key: Optional[str] = "messages"):
    def invoke_chain(state: AgentState):
        log(f"{invoke_chain.__name__} START.")

        messages = state[messages_key]

        output = chain.invoke({"messages": messages})

        messages += [output]
        log(f"{invoke_chain.__name__} END.")

    return invoke_chain


def should_invoke_tools(state: AgentState):
    log(f"{should_invoke_tools.__name__} START.")

    messages = state["messages"]

    last_message = messages[-1]

    log(f"{should_invoke_tools.__name__} END.")

    if last_message.tool_calls:
        return "invoke_tools"

    return "continue"


def invoke_tools(state: AgentState, tool_executor):
    log(f"{invoke_tools.__name__} START.")

    messages = state["messages"]

    last_message = messages[-1]
    tool_invocations = []

    for tool_call in last_message.tool_calls:
        action = ToolInvocation(
            tool=tool_call["name"],
            tool_input=tool_call["args"],
        )
        tool_invocations.append(action)

    responses = tool_executor.batch(tool_invocations, return_exceptions=True)

    tool_messages = [
        ToolMessage(
            content=str(response),
            name=tc["name"],
            tool_call_id=tc["id"],
        )
        for tc, response in zip(last_message.tool_calls, responses)
    ]

    log(f"{invoke_tools.__name__} END.")

    return {"messages": tool_messages}



from json import load as json_load
import os


def load_openai_config(config_file_path: str = ".config/openai_config.json"):
    with open(config_file_path) as f:
        openai_config = json_load(f)
        os.environ["AZURE_OPENAI_API_KEY"] = openai_config["openai_api_key"]
        os.environ["AZURE_OPENAI_ENDPOINT"] = openai_config["azure_endpoint"]
        os.environ["AZURE_OPENAI_API_VERSION"] = openai_config["openai_api_version"]
        return openai_config



from typing import List
import chromadb
import uuid
from chromadb.config import Settings
from langchain_chroma import Chroma
from langchain_openai import AzureOpenAIEmbeddings
from langchain_community.document_loaders import TextLoader
from langchain_text_splitters import (
    CharacterTextSplitter,
    RecursiveCharacterTextSplitter,
)
from .model_factory import EmbeddingFactory


class ChromaHttpClientFactory:
    @staticmethod
    def create():
        chroma_client = chromadb.HttpClient(
            settings=Settings(allow_reset=True), host="localhost", port=8000
        )
        return chroma_client


def embed_and_publish(file_paths, collection_name):
    documents = []
    collection = chroma_client.create_collection(collection_name)

    for file_path in file_paths:
        with open(file_path, "r", encoding="utf-8") as file:
            content = file.read()
            embedding = azure_embedding.embed_documents([content])[0]
            documents.append(
                {
                    "id": file_path,
                    "text": content,
                    "embedding": embedding,
                }
            )

    collection.add(documents=documents)


def chunk_embed_and_publish(
    file_paths: List[str],
    collection_name: str,
    embedding_function: AzureOpenAIEmbeddings,
    chroma_client: chromadb.HttpClient,
):
    vector_store = Chroma(
        embedding_function=embedding_function,
        client=chroma_client,
        collection_name=collection_name,
    )

    for file_path in file_paths:
        loader = TextLoader(file_path)
        docs = loader.load()
        text_splitter = RecursiveCharacterTextSplitter.from_tiktoken_encoder(
            chunk_size=1500, chunk_overlap=50
        )
        doc_splits = text_splitter.split_documents(docs)

        split_texts = [doc.page_content for doc in doc_splits]
        embeddings = embedding_function.embed_documents(split_texts)
        ids = [f"{file_path}_{i}" for i in range(len(split_texts))]

        vector_store.add_documents(documents=doc_splits, embeddings=embeddings, ids=ids)


def create_retriever(
    collection_name, chroma_client, embedding_function: AzureOpenAIEmbeddings
):
    vector_store = Chroma(
        embedding_function=embedding_function,
        collection_name=collection_name,
        client=chroma_client,
    )

    # retriever = vector_store.as_retriever(
    #     search_type="similarity", search_kwargs={"k": 5}
    # )
    retriever = vector_store.as_retriever()

    return retriever



from setuptools import setup, find_packages

# metadata...
name = "agnt_smth"
description = (
    "A simple agentic workflow builder framework, ontop of LangChain and LangGraph."
)
author = "Simon Stipcich"
author_email = "stipcich.simon@gmail.com"
url = "https://github.com/stiproot/agnt-smth"
license = "MIT"
keywords = ["python", "package", "langchain", "langgraph", "agentic", "beta"]
version = "0.1.8"

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

# dependencies...
install_requires = [
    "environs",
    "langchain",
    "langchain-community",
    "langchainhub",
    "langchain-chroma",
    "langchain-openai",
    "langchain-huggingface",
    "bs4",
    "chromadb",
    "sentence-transformers",
    "langgraph",
    "langsmith",
    "IPython",
    "pydantic",
    "rich",
]

# setup...
setup(
    name=name,
    version=version,
    packages=find_packages(where="src"),
    package_dir={"agnt_smth": "src/agnt_smth"},
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=author,
    author_email=author_email,
    url=url,
    license=license,
    keywords=keywords,
    install_requires=install_requires,
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.11",
    ],
)



TXT_TO_YML_SYSP = """ 
    You are work item translator assistant.\n
    You translate raw text into a specific work item tree structure, as a YAML file.\n
    Your output should only consist of YAML formatted text.\n
    Use the retrieve tool to search and return additional information about the correct structure of work items as YAML.\n
    """

YML_TO_JSON_SYSP = """ 
    You convert YML to JSON.\n
    Your output should only consist of JSON text.\n
    """



import operator
from typing import (
    Sequence,
    TypedDict,
    Annotated,
    Annotated,
)
from langchain_core.messages import BaseMessage


class AgentState(TypedDict):
    user_input: str
    global_messages: Annotated[Sequence[BaseMessage], operator.add]
    messages: Annotated[Sequence[BaseMessage], operator.add]


class C4ContextAgentState(AgentState):
    c4_context_diagram_path: str


class C4ContainerAgentState(AgentState):
    c4_context_diagram_path: str
    c4_container_diagram_path: str


class C4ComponentAgentState(AgentState):
    c4_container_diagram_path: str
    c4_component_diagram_path: str


class TaskTreeAgentState(AgentState):
    c4_component_diagram_path: str
    code_path: str


class RootState(TypedDict):
    user_input: str
    global_messages: Annotated[Sequence[BaseMessage], operator.add]
    c4_context_diagram_path: str
    c4_container_diagram_path: str
    c4_component_diagram_path: str
    code_path: str



from typing import Awaitable
from ..core.utls import log, traverse_folder


async def embed_file_system_contents(file_system_path: str) -> Awaitable:
    """Embeds the contents of a file system into a dictionary.

    Args:
      file_system_path: The path to the file system to embed.

    Returns:
      Awaitable: an empty promise.
    """

    log(f"embed_file_system_contents START. file_system_path: {file_system_path}")

    file_tree = traverse_folder(
        folder_path=file_system_path,
        ignore_folders=[".git", "__pycache__", "node_modules", "bin", "obj"],
    )

    log(f"file_tree: {file_tree}")